import tangram.BigNumber
import tangram.Log

// zoznam tovarov s aktualnou cenou (GET), query-parametre:
// [store=XXX] kod predajne (povinny)
// [minutes=nn] len zmenene zaznamy za poslednych n minut (ak sa neuvedie, bezi full-export)
urlPriceList (sql, request, json, cfg)

private static void urlPriceList (tangram.SQLconn sql, tangram.http.TangramHttpRequest request, tangram.TangramJsonWriter json, tangram.ConfigFile cfg)
{
    // Parametre z URL a konfiguracie (Tangram.ini)
    def store = request.queryMap.getString('store').toUpperCase()
    def minutes = request.queryMap.getInt('minutes')
    def censkup = cfg.str('EshopPriceGroup', '001')
//    def typceny = cfg.str('EshopPriceBasic', "'A','B'")

    def tovfilter = 'WHERE T.Neaktivna is null and (A.xx is not null or B.Xx is not null)'
    if (minutes) tovfilter = tovfilter + """
        and (exists (select null FROM TOVARY X WHERE X.Xx=T.Xx and nvl(X.Xud,X.Xid) > (sysdate - interval '$minutes' minute))
          or exists (select null FROM TOVCENY X WHERE X.Kod=T.Xx and nvl(X.Xud,X.Xid) > (sysdate - interval '$minutes' minute)))
    """
    // Log.debug ("store=$store, minutes=$minutes, filter=$tovfilter\ncena='$cenfilter'")
    // Nacitanie riadkov z tabulky TOVARY
    def select = """
    SELECT T.Xx, T.Nazov, T.Jmj, T.Jkoef, T.Povod, T.Pozn, T.Znacka, T.Zlozenie, T.Prodskup, T.Druh, T.Obal,
        PRICE.Cm_sdph "price", PRICE.Datum_od "DatBegin", PRICE.Datum_do "DatEnd", PRICE.Akcia "Akcia",
        BASIC.Cm_sdph "priceBasic", BASIC.Akcia "akciaBasic",
        PACK.Cm_sdph "pricePack",
        S.Nazov "nameOfOrigin"
    FROM TOVARY T 
    LEFT JOIN
    ( SELECT Kod, max(Xx) KEEP (DENSE_RANK FIRST ORDER BY Cm_sdph) as Xx FROM TOVCENY
      WHERE Censkup='$censkup' and (trunc(sysdate) between Datum_od and Datum_do) and (Typceny='B' or (Typceny='D' and Adresa='$store'))
      GROUP BY Kod
    ) A ON (A.Kod=T.Xx)
    LEFT JOIN
    ( SELECT Kod, max(Xx) KEEP (DENSE_RANK FIRST ORDER BY Cm_sdph) as Xx FROM TOVCENY
      WHERE Censkup='$censkup' and (trunc(sysdate) between Datum_od and Datum_do) and Typceny='A'
      GROUP BY Kod
    ) B ON (B.Kod=T.Xx)
    LEFT JOIN
    ( SELECT Kod, min(Cm_sdph) as Cm_sdph FROM TOVCENY
      WHERE Censkup='$censkup' and (trunc(sysdate) between Datum_od and Datum_do) and Typceny='A'
      GROUP BY Kod
    ) PACK ON (PACK.Kod=T.Obal)

    LEFT JOIN TOVCENY PRICE ON (PRICE.Xx=A.Xx)
    LEFT JOIN TOVCENY BASIC ON (BASIC.Xx=B.Xx)
    LEFT JOIN C_STATY S ON (S.Xx=T.Povod)
    $tovfilter
    ORDER BY T.Xx
    """

	Log.debug (select)

    def rows = sql.queryRows(select)

    // Response header
    request.sendHeaderJSON()

    // Reponse body
    def count = 0
    json.begin()
    json.arrayBegin('items')
    while (rows.next())
    {
        // dopocitane hodnoty
        def kod = rows.getString('xx')
        def basic = rows.getBigNumber('priceBasic')
        def price = rows.getBigNumber('price')
        def akcia = rows.getString('akcia')
        Log.debug("kod=$kod, basic=$basic, price=$price")
        def discount = new BigNumber()
        if (price.nullOrZero)
        {
            price = basic
            akcia = rows.getString('akciaBasic')
        }
        else 
		{
			if (! basic.nullOrZero) discount = ((basic - price) / basic * (new BigNumber(100))).round(1)
		}
        def jkoef = rows.getBigNumber('jkoef')
        def unitPrice = jkoef.nullOrZero ? new BigNumber() : (price / jkoef).round(2)

        // zapis do JSON
        json.blockBegin()

        json.pair('sku', kod)
        json.pair('store', store)
        json.pair('name', rows.nazov)
        json.pair('unit_price', unitPrice)
        json.pair('unit', rows.jmj)
        json.pair('place_of_origin', rows.povod)
        json.pair('name_of_origin', rows.nameOfOrigin)
        json.pair('price', price)
        json.pair('price_old', basic)
        json.pair('discount_value', discount)
        json.pair('description', rows.pozn)
        json.pair('brand', rows.znacka)
        json.pair('spec', rows.zlozenie)
        json.pair('typ_template', akcia)
        json.pair('date_begin', rows.getDate('datbegin'))
        json.pair('date_end', rows.getDate('datend'))
        json.pair('icon', rows.prodskup)
        json.pair('category', rows.druh)
        json.pair('grade', 'Trieda II.')
        json.pair("sku_pack", rows.obal)
        json.pair("rp_price", rows.pricePack)

        // pole ciarovych kodov
        exportBarcodes(sql, json, kod)

        // koniec bloku (jeden tovar)
        json.blockEnd()
        json.flush()
        count++
    }
    json.arrayEnd()
    json.end()

    rows.close()
    Log.info("Prices exported $count")
}

// export ciarovych kodov
private static void exportBarcodes (tangram.SQLconn sql, tangram.TangramJsonWriter json, String kod)
{
    // zoznam ciarovych kodov do Listu
    def barcodes = []
    def rows = sql.queryRows("SELECT Barcode FROM TOVBARCODE WHERE Kod='$kod' ORDER BY Barcode")
    while (rows.next()) barcodes.add(rows.barcode)
    rows.close()

    // export pola do JSON
    json.array(barcodes, "ean")
}
